@@ -120,6 +120,9 @@ gnl_composition_remove_object (GstBin * bin, GstElement * element);
 static GstStateChangeReturn
 gnl_composition_change_state (GstElement * element, GstStateChange transition);
 
+static GstPad * get_src_pad (GstElement * element);
+static void
+pad_blocked (GstPad * pad, gboolean blocked, GnlComposition * comp);
 
 static gboolean
 seek_handling (GnlComposition * comp, gboolean initial, gboolean update);
@@ -328,11 +331,45 @@ gnl_composition_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+unblock_child_pads (GstElement * child, GValue * ret, GnlComposition *comp)
+{
+  GstPad *pad;
+
+  GST_DEBUG_OBJECT (child, "unblocking pads");
+  pad = get_src_pad (child);
+  if (pad) {
+      gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked,
+				 comp);
+      gst_object_unref (pad);
+    }
+  gst_object_unref (child);
+  return TRUE;
+}
+
+static void
+unblock_childs (GnlComposition * comp)
+{
+  GstIterator *childs;
+  GstIteratorResult res;
+  GValue val = { 0 };
+
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&val, FALSE);
+  childs = gst_bin_iterate_elements (GST_BIN (comp));
+  res =
+      gst_iterator_fold (childs, (GstIteratorFoldFunction) unblock_child_pads,
+      &val, comp);
+  gst_iterator_free (childs);
+}
+
+
 static gboolean
 unlock_child_state (GstElement * child, GValue * ret, gpointer udata)
 {
   GST_DEBUG_OBJECT (child, "unlocking state");
   gst_element_set_locked_state (child, FALSE);
+  gst_object_unref (child);
   return TRUE;
 }
 
@@ -341,6 +378,7 @@ lock_child_state (GstElement * child, GValue * ret, gpointer udata)
 {
   GST_DEBUG_OBJECT (child, "locking state");
   gst_element_set_locked_state (child, TRUE);
+  gst_object_unref (child);
   return TRUE;
 }
 
@@ -477,6 +515,10 @@ gnl_composition_handle_message (GstBin * bin, GstMessage * message)
             "There was already a pending segment_done in main thread !");
         g_source_remove (comp->private->pending_idle);
       }
+
+      /* FIXME : This should be switched to using a g_thread_create() instead
+       * of a g_idle_add(). EXTENSIVE TESTING AND ANALYSIS REQUIRED BEFORE
+       * DOING THE SWITCH !!! */
       comp->private->pending_idle =
           g_idle_add ((GSourceFunc) segment_done_main_thread, (gpointer) comp);
 
@@ -496,7 +538,11 @@ gnl_composition_handle_message (GstBin * bin, GstMessage * message)
 static gint
 priority_comp (GnlObject * a, GnlObject * b)
 {
-  return a->priority - b->priority;
+  if (a->priority < b->priority)
+    return -1;
+  if (a->priority > b->priority)
+    return 1;
+  return 0;
 }
 
 static gboolean
@@ -527,10 +573,12 @@ have_to_update_pipeline (GnlComposition * comp)
  * before calling this function.
  */
 static GstEvent *
-get_new_seek_event (GnlComposition * comp, gboolean initial)
+get_new_seek_event (GnlComposition * comp, gboolean initial,
+		    gboolean updatestoponly)
 {
   GstSeekFlags flags;
   gint64 start, stop;
+  GstSeekType starttype = GST_SEEK_TYPE_SET;
 
   GST_DEBUG_OBJECT (comp, "initial:%d", initial);
   /* remove the seek flag */
@@ -540,19 +588,63 @@ get_new_seek_event (GnlComposition * comp, gboolean initial)
     flags =
         GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_ACCURATE | GST_SEEK_FLAG_FLUSH;
 
+  GST_DEBUG_OBJECT (comp, "private->segment->start:%"GST_TIME_FORMAT" segment_start%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (comp->private->segment->start),
+		    GST_TIME_ARGS (comp->private->segment_start));
+  GST_DEBUG_OBJECT (comp, "private->segment->stop:%"GST_TIME_FORMAT" segment_stop%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (comp->private->segment->stop),
+		    GST_TIME_ARGS (comp->private->segment_stop));
+
   start = MAX (comp->private->segment->start, comp->private->segment_start);
   stop = GST_CLOCK_TIME_IS_VALID (comp->private->segment->stop)
       ? MIN (comp->private->segment->stop, comp->private->segment_stop)
       : comp->private->segment_stop;
+  if (updatestoponly) {
+    starttype = GST_SEEK_TYPE_NONE;
+    start = GST_CLOCK_TIME_NONE;
+  }
 
   GST_DEBUG_OBJECT (comp,
       "Created new seek event. Flags:%d, start:%" GST_TIME_FORMAT ", stop:%"
       GST_TIME_FORMAT, flags, GST_TIME_ARGS (start), GST_TIME_ARGS (stop));
   return gst_event_new_seek (comp->private->segment->rate,
-      comp->private->segment->format, flags, GST_SEEK_TYPE_SET, start,
+      comp->private->segment->format, flags, starttype, start,
       GST_SEEK_TYPE_SET, stop);
 }
 
+static GstClockTime
+get_current_position (GnlComposition * comp)
+{
+  GstFormat format = GST_FORMAT_TIME;
+  gint64 value = GST_CLOCK_TIME_NONE;
+  GstPad *pad;
+  GnlObject *obj;
+  gboolean res;
+  
+  if (!comp->private->current) {
+    GST_WARNING_OBJECT (comp, "No current stack, can't send query");
+    goto beach;
+  }
+
+  obj = (GnlObject *) comp->private->current->data;
+
+  if (!(pad = get_src_pad ((GstElement*) obj)))
+    goto beach;
+
+  res = gst_pad_query_position (pad, &format, &value);
+
+  if (format != GST_FORMAT_TIME) {
+    GST_WARNING_OBJECT (comp, "query returned a format different from TIME");
+    value = GST_CLOCK_TIME_NONE;
+  } else {
+    GST_LOG_OBJECT (comp, "Query returned %"GST_TIME_FORMAT,
+		    GST_TIME_ARGS ((guint64) value));
+  }
+    
+ beach:
+  return (guint64) value;
+}
+
 /*
   Figures out if pipeline needs updating.
   Updates it and sends the seek event.
@@ -571,6 +663,9 @@ seek_handling (GnlComposition * comp, gboolean initial, gboolean update)
   comp->private->flushing = TRUE;
 
   /* Send downstream flush start/stop if needed */
+  /* FIXME : The problem is that we might have a streaming thread which will
+   * be able to push AFTER FLUSH_STOP.
+   * We can't really block... since maybe we're not pushing anything currently !!! */
   if (comp->private->ghostpad
       && (comp->private->segment->flags & GST_SEEK_FLAG_FLUSH)
       && (!update)) {
@@ -638,7 +733,9 @@ gnl_composition_event_handler (GstPad * ghostpad, GstEvent * event)
 
       /* the incoming event might not be quite correct, we get a new proper
        * event to pass on to the childs. */
-      nevent = get_new_seek_event (comp, FALSE);
+      COMP_OBJECTS_LOCK (comp);
+      nevent = get_new_seek_event (comp, FALSE, FALSE);
+      COMP_OBJECTS_UNLOCK (comp);
       gst_event_unref (event);
       event = nevent;
       break;
@@ -726,41 +823,63 @@ gnl_composition_ghost_pad_set_target (GnlComposition * comp, GstPad * target)
   GST_DEBUG_OBJECT (comp, "END");
 }
 
-static GstClockTime
-next_stop_in_region_above_priority (GnlComposition * composition,
-    GstClockTime start, GstClockTime stop, guint priority)
+static void
+refine_start_stop_in_region_above_priority (GnlComposition * composition,
+					    GstClockTime timestamp, GstClockTime start,
+					    GstClockTime stop,
+					    GstClockTime * rstart, GstClockTime * rstop,
+					    guint32 priority)
 {
   GList *tmp = composition->private->objects_start;
   GnlObject *object;
-  GstClockTime res = stop;
+  GstClockTime nstart = start, nstop = stop;
 
-  GST_DEBUG_OBJECT (composition, "start: %" GST_TIME_FORMAT " stop: %"
-      GST_TIME_FORMAT " priority:%d",
+  GST_DEBUG_OBJECT (composition, "timestamp:%" GST_TIME_FORMAT" start: %" GST_TIME_FORMAT " stop: %"
+		    GST_TIME_FORMAT " priority:%u", GST_TIME_ARGS (timestamp),
       GST_TIME_ARGS (start), GST_TIME_ARGS (stop), priority);
 
-  /* Find first object with priority < priority, and start < start_position <= stop */
-  for (; tmp; tmp = g_list_next (tmp)) {
+  for (tmp = composition->private->objects_start; tmp; tmp = g_list_next (tmp)) {
     object = (GnlObject *) tmp->data;
 
-    if (object->priority >= priority)
+    GST_LOG_OBJECT (object, "START %"GST_TIME_FORMAT"--%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (object->start), GST_TIME_ARGS (object->stop));
+
+    if ((object->priority >= priority) || (!object->active))
       continue;
 
-    if (object->stop <= start)
+    if (object->start <= timestamp)
       continue;
 
-    if (object->start <= start)
-      continue;
-
-    if (object->start > stop)
-      break;
-
-    res = object->start;
-    GST_DEBUG_OBJECT (composition, "Found %s [prio:%d] at %" GST_TIME_FORMAT,
-        GST_OBJECT_NAME (object),
-        object->priority, GST_TIME_ARGS (object->start));
+    nstop = object->start;
+    GST_DEBUG_OBJECT (composition, "START Found %s [prio:%u] at %" GST_TIME_FORMAT,
+		      GST_OBJECT_NAME (object),
+		      object->priority, GST_TIME_ARGS (object->start));
     break;
   }
-  return res;
+
+  for (tmp = composition->private->objects_stop; tmp; tmp = g_list_next (tmp)) {
+    object = (GnlObject *) tmp->data;
+
+    GST_LOG_OBJECT (object, "STOP %"GST_TIME_FORMAT"--%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (object->start), GST_TIME_ARGS (object->stop));
+
+    if ((object->priority >= priority) || (!object->active))
+      continue;
+
+    if (object->stop >= timestamp)
+      continue;
+
+    nstart = object->stop;
+    GST_DEBUG_OBJECT (composition, "STOP Found %s [prio:%u] at %" GST_TIME_FORMAT,
+		      GST_OBJECT_NAME (object),
+		      object->priority, GST_TIME_ARGS (object->start));
+    break;
+  }
+
+  if (*rstart)
+    *rstart = nstart;
+  if (*rstop)
+    *rstop = nstop;
 }
 
 
@@ -773,7 +892,8 @@ next_stop_in_region_above_priority (GnlComposition * composition,
  */
 
 static GNode *
-convert_list_to_tree (GList ** stack, GstClockTime * stop, guint * highprio)
+convert_list_to_tree (GList ** stack, GstClockTime * start,
+		      GstClockTime * stop, guint32 * highprio)
 {
   GNode *ret;
   guint nbsinks;
@@ -786,8 +906,10 @@ convert_list_to_tree (GList ** stack, GstClockTime * stop, guint * highprio)
 
   object = (GnlObject *) (*stack)->data;
 
-  GST_DEBUG ("object:%s , stop:%" GST_TIME_FORMAT "highprio:%d",
-      GST_ELEMENT_NAME (object), GST_TIME_ARGS (*stop), *highprio);
+  GST_DEBUG ("object:%s , *start:%"GST_TIME_FORMAT", *stop:%"
+	     GST_TIME_FORMAT " highprio:%d",
+	     GST_ELEMENT_NAME (object), GST_TIME_ARGS (*start), 
+	     GST_TIME_ARGS (*stop), *highprio);
 
   /* update earliest stop */
   if (GST_CLOCK_TIME_IS_VALID (*stop)) {
@@ -797,6 +919,13 @@ convert_list_to_tree (GList ** stack, GstClockTime * stop, guint * highprio)
     *stop = object->stop;
   }
 
+  if (GST_CLOCK_TIME_IS_VALID (*start)) {
+    if (GST_CLOCK_TIME_IS_VALID (object->start) && (*start < object->start))
+      *start = object->start;
+  } else {
+    *start = object->start;
+  }
+
   if (GNL_IS_SOURCE (object)) {
     *stack = g_list_next (*stack);
     /* update highest priority.
@@ -816,7 +945,7 @@ convert_list_to_tree (GList ** stack, GstClockTime * stop, guint * highprio)
     /* FIXME : if num_sinks == -1 : request the proper number of pads */
 
     for (tmp = g_list_next (*stack); tmp && (!limit || nbsinks);) {
-      g_node_append (ret, convert_list_to_tree (&tmp, stop, highprio));
+      g_node_append (ret, convert_list_to_tree (&tmp, start, stop, highprio));
 
       if (limit)
         nbsinks--;
@@ -824,8 +953,8 @@ convert_list_to_tree (GList ** stack, GstClockTime * stop, guint * highprio)
   }
 
 beach:
-  GST_DEBUG_OBJECT (object, "*stop:%" GST_TIME_FORMAT " priority:%d",
-      GST_TIME_ARGS (*stop), *highprio);
+  GST_DEBUG_OBJECT (object, "*start:%"GST_TIME_FORMAT " *stop:%" GST_TIME_FORMAT " priority:%u",
+		    GST_TIME_ARGS (*start), GST_TIME_ARGS (*stop), *highprio);
 
   return ret;
 }
@@ -836,6 +965,7 @@ beach:
  * @timestamp: The #GstClockTime to look at
  * @priority: The priority level to start looking from
  * @activeonly: Only look for active elements if TRUE
+ * @start: The biggest start time of the objects in the stack
  * @stop: The smallest stop time of the objects in the stack
  * @highprio: The highest priority in the stack
  *
@@ -846,24 +976,29 @@ beach:
 
 static GNode *
 get_stack_list (GnlComposition * comp, GstClockTime timestamp,
-    guint priority, gboolean activeonly, GstClockTime * stop, guint * highprio)
+		guint32 priority, gboolean activeonly, GstClockTime * start,
+		GstClockTime * stop, guint * highprio)
 {
   GList *tmp = comp->private->objects_start;
   GList *stack = NULL;
   GNode *ret = NULL;
+  GstClockTime nstart = GST_CLOCK_TIME_NONE;
   GstClockTime nstop = GST_CLOCK_TIME_NONE;
-  guint highest = 0;
+  guint32 highest = 0;
 
   GST_DEBUG_OBJECT (comp,
-      "timestamp:%" GST_TIME_FORMAT ", priority:%d, activeonly:%d",
+      "timestamp:%" GST_TIME_FORMAT ", priority:%u, activeonly:%d",
       GST_TIME_ARGS (timestamp), priority, activeonly);
 
+  GST_LOG ("objects_start:%p",
+	   comp->private->objects_start);
+
   for (; tmp; tmp = g_list_next (tmp)) {
     GnlObject *object = (GnlObject *) tmp->data;
 
     GST_LOG_OBJECT (object,
         "start: %" GST_TIME_FORMAT " , stop:%" GST_TIME_FORMAT " , duration:%"
-        GST_TIME_FORMAT ", priority:%d", GST_TIME_ARGS (object->start),
+        GST_TIME_FORMAT ", priority:%u", GST_TIME_ARGS (object->start),
         GST_TIME_ARGS (object->stop), GST_TIME_ARGS (object->duration),
         object->priority);
 
@@ -888,9 +1023,16 @@ get_stack_list (GnlComposition * comp, GstClockTime timestamp,
 
   /* convert that list to a stack */
   tmp = stack;
-  ret = convert_list_to_tree (&tmp, &nstop, &highest);
-  if (nstop)
+  ret = convert_list_to_tree (&tmp, &nstart, &nstop, &highest);
+
+  GST_DEBUG ("nstart:%"GST_TIME_FORMAT", nstop:%"GST_TIME_FORMAT,
+	     GST_TIME_ARGS (nstart),
+	     GST_TIME_ARGS (nstop));
+
+  if (*stop)
     *stop = nstop;
+  if (*start)
+    *start = nstart;
   if (highprio)
     *highprio = highest;
 
@@ -904,23 +1046,29 @@ get_stack_list (GnlComposition * comp, GstClockTime timestamp,
  * @comp: The #GnlComposition
  * @timestamp: The #GstClockTime to look at
  * @stop_time: Pointer to a #GstClockTime for min stop time of returned stack
+ * @start_time: Pointer to a #GstClockTime for greatest start time of returned stack
  *
  * Returns: The new current stack for the given #GnlComposition and @timestamp.
  */
 
 static GNode *
 get_clean_toplevel_stack (GnlComposition * comp, GstClockTime * timestamp,
-    GstClockTime * stop_time)
+			  GstClockTime * start_time, GstClockTime * stop_time)
 {
   GNode *stack = NULL;
   GList *tmp;
+  GstClockTime start = G_MAXUINT64;
   GstClockTime stop = G_MAXUINT64;
   guint highprio;
 
   GST_DEBUG_OBJECT (comp, "timestamp:%" GST_TIME_FORMAT,
       GST_TIME_ARGS (*timestamp));
 
-  stack = get_stack_list (comp, *timestamp, 0, TRUE, &stop, &highprio);
+  stack = get_stack_list (comp, *timestamp, 0, TRUE, &start, &stop, &highprio);
+
+  GST_DEBUG ("start:%"GST_TIME_FORMAT", stop:%"GST_TIME_FORMAT,
+	     GST_TIME_ARGS (start),
+	     GST_TIME_ARGS (stop));
 
   if (!stack) {
     GnlObject *object = NULL;
@@ -943,30 +1091,45 @@ get_clean_toplevel_stack (GnlComposition * comp, GstClockTime * timestamp,
           GST_TIME_FORMAT "]", GST_TIME_ARGS (*timestamp),
           GST_ELEMENT_NAME (object), GST_TIME_ARGS (object->start));
       *timestamp = object->start;
-      stack = get_stack_list (comp, *timestamp, 0, TRUE, &stop, &highprio);
+      stack = get_stack_list (comp, *timestamp, 0, TRUE, &start, &stop, &highprio);
     }
   }
 
+  GST_DEBUG ("start:%"GST_TIME_FORMAT", stop:%"GST_TIME_FORMAT,
+	     GST_TIME_ARGS (start),
+	     GST_TIME_ARGS (stop));
+
   if (stack) {
     guint32 top_priority;
 
     top_priority = GNL_OBJECT (stack->data)->priority;
 
     /* Figure out if there's anything blocking us with smaller priority */
-    stop =
-        next_stop_in_region_above_priority (comp, *timestamp, stop,
-        (highprio == 0) ? top_priority : highprio);
+    refine_start_stop_in_region_above_priority (comp, *timestamp, start, stop,
+						&start, &stop,
+						(highprio == 0) ? top_priority : highprio);
   }
-
-  if (stop_time) {
+  
+  if (*stop_time) {
     if (stack)
       *stop_time = stop;
     else
       *stop_time = 0;
-    GST_LOG_OBJECT (comp, "Setting stop_time to %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (*stop_time));
   }
 
+  if (*start_time) {
+    if (stack)
+      *start_time = start;
+    else
+      *start_time = 0;
+  }
+
+  GST_DEBUG_OBJECT (comp, "Returning timestamp:%"GST_TIME_FORMAT" , start_time:%"
+		    GST_TIME_FORMAT" , stop_time:%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (*timestamp),
+		    GST_TIME_ARGS (*start_time),
+		    GST_TIME_ARGS (*stop_time));
+
   return stack;
 }
 
@@ -1061,6 +1224,10 @@ gnl_composition_change_state (GstElement * element, GstStateChange transition)
     return ret;
 
   switch (transition) {
+  case GST_STATE_CHANGE_PAUSED_TO_READY:
+  case GST_STATE_CHANGE_READY_TO_NULL:
+    unblock_childs(comp);
+    break;
     default:
       break;
   }
@@ -1072,8 +1239,13 @@ beach:
 static gint
 objects_start_compare (GnlObject * a, GnlObject * b)
 {
-  if (a->start == b->start)
-    return a->priority - b->priority;
+  if (a->start == b->start) {
+    if (a->priority < b->priority)
+      return -1;
+    if (a->priority > b->priority)
+      return 1;
+    return 0;
+  }
   if (a->start < b->start)
     return -1;
   if (a->start > b->start)
@@ -1084,8 +1256,13 @@ objects_start_compare (GnlObject * a, GnlObject * b)
 static gint
 objects_stop_compare (GnlObject * a, GnlObject * b)
 {
-  if (a->stop == b->stop)
-    return a->priority - b->priority;
+  if (a->stop == b->stop) {
+    if (a->priority < b->priority)
+      return -1;
+    if (a->priority > b->priority)
+      return 1;
+    return 0;
+  }
   if (b->stop < a->stop)
     return -1;
   if (b->stop > a->stop)
@@ -1166,10 +1343,11 @@ no_more_pads_object_cb (GstElement * element, GnlComposition * comp)
   GnlObject *object = GNL_OBJECT (element);
   GNode *tmp;
   GstPad *pad = NULL;
+  GstPad *tpad = NULL;
 
   GST_LOG_OBJECT (element, "no more pads");
   if (!(pad = get_src_pad (element)))
-    return;
+    goto no_source;
 
   COMP_OBJECTS_LOCK (comp);
 
@@ -1197,7 +1375,7 @@ no_more_pads_object_cb (GstElement * element, GnlComposition * comp)
     }
 
     if (comp->private->current && comp->private->waitingpads == 0) {
-      GstPad *tpad = get_src_pad (GST_ELEMENT (comp->private->current->data));
+      tpad = get_src_pad (GST_ELEMENT (comp->private->current->data));
 
       /* There are no more waiting pads for the currently configured timeline */
       /* stack. */
@@ -1234,8 +1412,20 @@ no_more_pads_object_cb (GstElement * element, GnlComposition * comp)
 done:
   COMP_OBJECTS_UNLOCK (comp);
 
+  if (pad)
+    gst_object_unref (pad);
+  if (tpad)
+    gst_object_unref (tpad);
 
   GST_DEBUG_OBJECT (comp, "end");
+
+  return;
+
+no_source:
+  {
+    GST_LOG_OBJECT (comp, "no source pad");
+    return;
+  }
 }
 
 /*
@@ -1257,7 +1447,7 @@ compare_relink_single_node (GnlComposition * comp, GNode * node,
   GnlObject *newobj;
   GnlObject *newparent;
   GnlObject *oldparent = NULL;
-  GstPad *srcpad;
+  GstPad *srcpad = NULL;
 
   if (!node)
     return;
@@ -1324,6 +1514,9 @@ compare_relink_single_node (GnlComposition * comp, GNode * node,
           G_CALLBACK (no_more_pads_object_cb), comp);
   }
 
+  if (srcpad)
+    gst_object_unref (srcpad);
+
   GST_LOG_OBJECT (newobj, "DONE");
 }
 
@@ -1522,6 +1715,50 @@ unlock_activate_stack (GnlComposition * comp, GNode * node,
     unlock_activate_stack (comp, child, change_state, state);
 }
 
+static gboolean
+are_same_stacks (GNode *stack1, GNode *stack2)
+{
+  gboolean res = FALSE;
+
+  /* TODO : FIXME : we should also compare start/media-start */
+
+  /* stacks are not equal if one of them is NULL but not the other */
+  if ((!stack1 && stack2) || (stack1 && !stack2))
+    goto beach;
+
+  if (stack1 && stack2) {
+    GNode *child1, *child2;
+
+    /* if they don't contain the same source, not equal */
+    if (!(stack1->data == stack2->data))
+      goto beach;
+
+    /* if they don't have the same number of childs, not equal */
+    if (!(g_node_n_children(stack1) == g_node_n_children(stack2)))
+      goto beach;
+
+    child1 = stack1->children;
+    child2 = stack2->children;
+    while (child1 && child2) {
+      if (!(are_same_stacks (child1, child2)))
+	goto beach;
+      child1 = g_node_next_sibling (child1);
+      child2 = g_node_next_sibling (child2);
+    }
+    
+    /* if there's a difference in child number, stacks are not equal */
+    if (child1 || child2)
+      goto beach;
+  }
+
+  /* if stack1 AND stack2 are NULL, then they're equal (both empty) */
+  res = TRUE;
+
+ beach:
+  GST_LOG ("Stacks are equal : %d", res);
+  return res;
+}
+
 /**
  * update_pipeline:
  * @comp: The #GnlComposition
@@ -1557,17 +1794,26 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
         (GST_STATE_NEXT (comp) ==
         GST_STATE_VOID_PENDING) ? GST_STATE (comp) : GST_STATE_NEXT (comp);
     GNode *stack = NULL;
-    GList *deactivate;
-    GstClockTime new_stop;
+    GList *deactivate = NULL;
+    GstClockTime new_start = GST_CLOCK_TIME_NONE;
+    GstClockTime new_stop = GST_CLOCK_TIME_NONE;
     gboolean switchtarget = FALSE;
+    gboolean samestack = FALSE;
+    gboolean startchanged, stopchanged;
 
     GST_DEBUG_OBJECT (comp,
         "now really updating the pipeline, current-state:%s",
         gst_element_state_get_name (state));
 
     /* (re)build the stack and relink new elements */
-    stack = get_clean_toplevel_stack (comp, &currenttime, &new_stop);
-    deactivate = compare_relink_stack (comp, stack, modify);
+    stack = get_clean_toplevel_stack (comp, &currenttime, &new_start, &new_stop);
+    samestack = are_same_stacks (comp->private->current, stack);
+
+    if (!samestack)
+      deactivate = compare_relink_stack (comp, stack, modify);
+
+    startchanged = comp->private->segment_start != currenttime;
+    stopchanged = comp->private->segment_stop != new_stop;
 
     /* set new segment_start/stop */
     comp->private->segment_start = currenttime;
@@ -1612,7 +1858,7 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
       g_node_destroy (comp->private->current);
     comp->private->current = stack;
 
-    if (stack)
+    if (!samestack && stack)
       unlock_activate_stack (comp, stack, change_state, nextstate);
     GST_DEBUG_OBJECT (comp, "Finished activating objects in new stack");
 
@@ -1624,7 +1870,10 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
       COMP_OBJECTS_LOCK (comp);
 
       /* 1. Create new seek event for newly configured timeline stack */
-      event = get_new_seek_event (comp, initial);
+      if (samestack && stopchanged)
+	event = get_new_seek_event (comp, (state == GST_STATE_PLAYING) ? FALSE : TRUE, TRUE);
+      else
+	event = get_new_seek_event (comp, initial, FALSE);
 
       /* 2. Is the stack entirely ready ? */
       if (comp->private->waitingpads == 0) {
@@ -1653,9 +1902,8 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
             GST_LOG_OBJECT (comp, "About to unblock top-level srcpad");
             gst_pad_set_blocked_async (pad, FALSE,
                 (GstPadBlockCallback) pad_blocked, comp);
-            gst_object_unref (pad);
           }
-
+          gst_object_unref (pad);
         } else {
           GST_WARNING_OBJECT (comp,
               "Timeline is entirely linked, but couldn't get top-level element's source pad");
@@ -1691,7 +1939,9 @@ static void
 object_start_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
+  GST_DEBUG_OBJECT (object, "start position changed (%"GST_TIME_FORMAT
+		    "), evaluating pipeline update",
+		    GST_TIME_ARGS (object->start));
 
   comp->private->objects_start = g_list_sort
       (comp->private->objects_start, (GCompareFunc) objects_start_compare);
@@ -1699,9 +1949,10 @@ object_start_changed (GnlObject * object, GParamSpec * arg,
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
 
-  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object)) {
+    comp->private->segment->start = comp->private->segment_start;
     update_pipeline (comp, comp->private->segment_start, TRUE, TRUE, TRUE);
-  else
+  } else
     update_start_stop_duration (comp);
 }
 
@@ -1709,7 +1960,9 @@ static void
 object_stop_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
+  GST_DEBUG_OBJECT (object, "stop position changed (%"GST_TIME_FORMAT
+		    "), evaluating pipeline update",
+		    GST_TIME_ARGS (object->stop));
 
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
@@ -1717,9 +1970,10 @@ object_stop_changed (GnlObject * object, GParamSpec * arg,
   comp->private->objects_start = g_list_sort
       (comp->private->objects_start, (GCompareFunc) objects_start_compare);
 
-  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object)) {
+    comp->private->segment->start = comp->private->segment_start;
     update_pipeline (comp, comp->private->segment_start, TRUE, TRUE, TRUE);
-  else
+  } else
     update_start_stop_duration (comp);
 }
 
@@ -1727,7 +1981,8 @@ static void
 object_priority_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
+  GST_DEBUG_OBJECT (object, "priority changed (%u), evaluating pipeline update",
+		    object->priority);
 
   comp->private->objects_start = g_list_sort
       (comp->private->objects_start, (GCompareFunc) objects_start_compare);
@@ -1735,9 +1990,10 @@ object_priority_changed (GnlObject * object, GParamSpec * arg,
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
 
-  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object)) {
+    comp->private->segment->start = comp->private->segment_start;
     update_pipeline (comp, comp->private->segment_start, TRUE, TRUE, TRUE);
-  else
+  } else
     update_start_stop_duration (comp);
 }
 
@@ -1745,13 +2001,15 @@ static void
 object_active_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
+  GST_DEBUG_OBJECT (object, "active flag changed (%d), evaluating pipeline update",
+		    object->active);
 
   update_start_stop_duration (comp);
 
-  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object)) {
+    comp->private->segment->start = comp->private->segment_start;
     update_pipeline (comp, comp->private->segment_start, TRUE, TRUE, TRUE);
-  else
+  } else
     update_start_stop_duration (comp);
 }
 
@@ -1766,6 +2024,10 @@ object_pad_removed (GnlObject * object, GstPad * pad, GnlComposition * comp)
     GST_DEBUG_OBJECT (comp, "Removing ghostpad");
     gnl_object_remove_ghost_pad (GNL_OBJECT (comp), comp->private->ghostpad);
     comp->private->ghostpad = NULL;
+  } else {
+    /* unblock it ! */
+    gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked,
+			       comp);
   }
 }
 
@@ -1785,6 +2047,7 @@ gnl_composition_add_object (GstBin * bin, GstElement * element)
   gboolean ret;
   GnlCompositionEntry *entry;
   GnlComposition *comp = GNL_COMPOSITION (bin);
+  GstClockTime curpos;
 
   GST_DEBUG_OBJECT (bin, "element %s", GST_OBJECT_NAME (element));
 
@@ -1813,6 +2076,10 @@ gnl_composition_add_object (GstBin * bin, GstElement * element)
     goto chiringuito;
   }
 
+  /* lock state of child ! */
+  GST_LOG_OBJECT (bin, "Locking state of %s", GST_ELEMENT_NAME (element));
+  gst_element_set_locked_state (element, TRUE);
+
   /* wrap new element in a GnlCompositionEntry ... */
   entry = g_new0 (GnlCompositionEntry, 1);
   entry->object = GNL_OBJECT (element);
@@ -1881,12 +2148,16 @@ gnl_composition_add_object (GstBin * bin, GstElement * element)
       GST_TIME_ARGS (comp->private->segment_start),
       GST_TIME_ARGS (comp->private->segment_stop));
 
+  curpos = get_current_position (comp);
+  if (curpos == GST_CLOCK_TIME_NONE)
+    curpos = comp->private->segment_start;
+
   COMP_OBJECTS_UNLOCK (comp);
 
   /* If we added within currently configured segment OR the pipeline was *
    * previously empty, THEN update pipeline */
   if (OBJECT_IN_ACTIVE_SEGMENT (comp, element) || (!comp->private->current))
-    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE, TRUE);
+    update_pipeline (comp, curpos, TRUE, TRUE, TRUE);
   else
     update_start_stop_duration (comp);
 
@@ -1953,6 +2224,17 @@ gnl_composition_remove_object (GstBin * bin, GstElement * element)
   GST_LOG_OBJECT (element, "Done removing from the composition");
 
 beach:
+  /* unblock source pad */
+  if (1) {
+    GstPad * pad = get_src_pad (element);
+
+    if (pad) {
+      gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked,
+				 comp);
+      gst_object_unref (pad);
+    }
+  }
+
   gst_object_unref (element);
   return ret;
 
