@@ -123,8 +123,11 @@ gnl_filesource_init (GnlFileSource * filesource, GnlFileSourceClass * klass)
             gst_element_factory_make ("filesrc", "internal-filesource")))
       g_warning
           ("Could not create a gnomevfssrc or filesource element, are you sure you have any of them installed ?");
-  if (!(decodebin =
-          gst_element_factory_make ("decodebin", "internal-decodebin")))
+  if (g_getenv ("USE_DECODEBIN2"))
+    decodebin = gst_element_factory_make ("decodebin2", "internal-decodebin");
+  else 
+    decodebin = gst_element_factory_make ("decodebin", "internal-decodebin");
+  if (!decodebin)
     g_warning
         ("Could not create a decodebin element, are you sure you have decodebin installed ?");
 
