@@ -202,7 +202,7 @@ gnl_object_dispose (GObject * object)
  * Returns: TRUE if @objecttime was within the limits of the @object start/stop time,
  * FALSE otherwise
  */
-static gboolean
+gboolean
 gnl_object_to_media_time (GnlObject * object, GstClockTime otime,
     GstClockTime * mtime)
 {
@@ -261,7 +261,7 @@ gnl_object_to_media_time (GnlObject * object, GstClockTime otime,
  * FALSE otherwise
  */
 
-static gboolean
+gboolean
 gnl_media_to_object_time (GnlObject * object, GstClockTime mtime,
     GstClockTime * otime)
 {
@@ -394,6 +394,7 @@ translate_incoming_seek (GnlObject * object, GstEvent * event)
   gdouble rate, nrate;
   GstSeekFlags flags;
   GstSeekType curtype, stoptype;
+  GstSeekType ncurtype;
   gint64 cur;
   guint64 ncur;
   gint64 stop;
@@ -415,15 +416,20 @@ translate_incoming_seek (GnlObject * object, GstEvent * event)
   GST_DEBUG ("nrate:%f , rate:%f, object->rate:%f", nrate, rate, object->rate);
 
   /* convert cur */
+  ncurtype = GST_SEEK_TYPE_SET;
   if ((curtype == GST_SEEK_TYPE_SET)
       && (gnl_object_to_media_time (object, cur, &ncur))) {
     if (ncur > G_MAXINT64)
       GST_WARNING_OBJECT (object, "return value too big...");
     GST_LOG_OBJECT (object, "Setting cur to %" GST_TIME_FORMAT,
         GST_TIME_ARGS (ncur));
-  } else {
+  } else if ((curtype != GST_SEEK_TYPE_NONE)) {
     GST_DEBUG_OBJECT (object, "Limiting seek start to media_start");
     ncur = object->media_start;
+  } else {
+    GST_DEBUG_OBJECT (object, "leaving GST_SEEK_TYPE_NONE");
+    ncur = cur;
+    ncurtype = GST_SEEK_TYPE_NONE;
   }
 
   /* convert stop, we also need to limit it to object->stop */
@@ -463,12 +469,12 @@ translate_incoming_seek (GnlObject * object, GstEvent * event)
 
 
   GST_DEBUG_OBJECT (object,
-      "SENDING SEEK rate:%f, format:TIME, flags:%d, curtype:SET, stoptype:SET, %"
-      GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, nrate, flags,
-      GST_TIME_ARGS (ncur), GST_TIME_ARGS (nstop));
+      "SENDING SEEK rate:%f, format:TIME, flags:%d, curtype:%d, stoptype:SET, %"
+		    GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, nrate, flags, ncurtype,
+		    GST_TIME_ARGS (ncur), GST_TIME_ARGS (nstop));
 
   event2 = gst_event_new_seek (nrate, GST_FORMAT_TIME, flags,
-      GST_SEEK_TYPE_SET, (gint64) ncur, GST_SEEK_TYPE_SET, (gint64) nstop);
+      ncurtype, (gint64) ncur, GST_SEEK_TYPE_SET, (gint64) nstop);
 
   gst_event_unref (event);
 
@@ -654,6 +660,12 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
       if (GST_EVENT_TYPE (event) == GST_EVENT_SEEK) {
 
         event = translate_incoming_seek (object, event);
+      } else if (GST_EVENT_TYPE (event) == GST_EVENT_QOS) {
+
+	/* FIXME : Implement proper QoS time-shifting, for the time being it's
+	 * just ignored. See #398453 */
+	gst_event_unref (event);
+	goto beach;
       }
       break;
     default:
@@ -665,6 +677,7 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
   GST_DEBUG_OBJECT (ghostpad, "Returned from calling priv->eventfunc : %d",
       ret);
 
+ beach:
   return ret;
 
   /* ERRORS */
@@ -725,6 +738,7 @@ control_internal_pad (GstPad * ghostpad, GnlObject * object)
   GST_LOG_OBJECT (ghostpad, "overriding ghostpad's internal pad function");
 
   internal = gst_pad_get_peer (target);
+  gst_object_unref (target);
 
   if (!(priv = gst_pad_get_element_private (internal))) {
     GST_DEBUG_OBJECT (internal,
@@ -789,7 +803,6 @@ gnl_object_ghost_pad_full (GnlObject * object, const gchar * name,
   if (ghost && (!(gnl_object_ghost_pad_set_target (object, ghost, target)))) {
     GST_WARNING_OBJECT (object,
         "Couldn't set the target pad... removing ghostpad");
-    gst_element_remove_pad (GST_ELEMENT (object), ghost);
     gst_object_unref (ghost);
     return NULL;
   }
@@ -861,6 +874,7 @@ gnl_object_remove_ghost_pad (GnlObject * object, GstPad * ghost)
   GST_DEBUG_OBJECT (object, "ghostpad %s:%s", GST_DEBUG_PAD_NAME (ghost));
 
   priv = gst_pad_get_element_private (ghost);
+  gst_ghost_pad_set_target (GST_GHOST_PAD (ghost), NULL);
   gst_element_remove_pad (GST_ELEMENT (object), ghost);
   if (priv)
     g_free (priv);
