@@ -128,6 +128,11 @@ gnl_source_init (GnlSource * source, GnlSourceClass * klass)
   GST_OBJECT_FLAG_SET (source, GNL_OBJECT_SOURCE);
   source->element = NULL;
   source->priv = g_new0 (GnlSourcePrivate, 1);
+
+  if (g_object_class_find_property (G_OBJECT_CLASS (parent_class), "async-handling")) {
+    GST_DEBUG_OBJECT (source, "Setting GstBin async-handling to TRUE");
+    g_object_set (G_OBJECT (source), "async-handling", TRUE, NULL);
+  }
 }
 
 static void
@@ -287,7 +292,7 @@ get_valid_src_pad (GnlSource * source, GstElement * element, GstPad ** pad)
   return FALSE;
 }
 
-static gboolean
+static gpointer
 ghost_seek_pad (GnlSource * source)
 {
   GstPad *pad = source->priv->ghostedpad;
@@ -319,10 +324,8 @@ ghost_seek_pad (GnlSource * source)
 
   source->priv->pendingblock = FALSE;
 
-  gst_object_unref (pad);
-
 beach:
-  return FALSE;
+  return NULL;
 }
 
 static void
@@ -333,7 +336,7 @@ pad_blocked_cb (GstPad * pad, gboolean blocked, GnlSource * source)
 
   if (!(source->priv->ghostpad)) {
     if (blocked)
-      g_idle_add ((GSourceFunc) ghost_seek_pad, source);
+      g_thread_create ((GThreadFunc) ghost_seek_pad, source, FALSE, NULL);
   }
 }
 
